
#include <QWidget>
#include <QString>
#include <QStringList>
#include <QGridLayout>
#include <QGroupBox>
#include <QCheckBox>

#include "enum.bactypes.h"
#include "BitString.h"

BitString::BitString(QWidget *parent, int db_id, int PropId, QString label, QGridLayout *layout, int row, int col)
  : Field(parent, db_id, PropId, BT_BIT_STR, label) {
  this->value = QVariant();
  bs = 0;
  sigmap = new QSignalMapper(this);

  box = new QGroupBox(label, parent);
  this->input = box;

  if(layout) {
    layout->addWidget(this->label, row, col, Qt::AlignRight);
    layout->addWidget(this->input, row, col + 1, Qt::AlignLeft);
  }
  checkCount = 0;
  checkLayout = new QGridLayout();
  checkLayout->setAlignment(Qt::AlignHCenter);
  box->setLayout(checkLayout);
  connect(sigmap, SIGNAL(mapped(int)), this, SLOT(flipBit(int)));
}

BitString::BitString(QWidget *parent, int db_id, int PropId, int value, QString label, QGridLayout *layout, int row, int col)
  : Field(parent, db_id, PropId, BT_BIT_STR, label) {
  this->value = bs = value;
  sigmap = new QSignalMapper(this);

  box = new QGroupBox(label, parent);
  this->input = box;

  if(layout) {
    layout->addWidget(this->label, row, col, Qt::AlignRight);
    layout->addWidget(this->input, row, col + 1, Qt::AlignLeft);
  }
  checkCount = 0;
  checkLayout = new QGridLayout();
  checkLayout->setAlignment(Qt::AlignHCenter);
  box->setLayout(checkLayout);
  connect(sigmap, SIGNAL(mapped(int)), this, SLOT(flipBit(int)));
}

void BitString::flipBit(int b) {
  bs ^= (1 << b);
  value = bs;
}

void BitString::SetValue(QVariant value) {
  int i, tmp;

 // Note: if any of these setCheckState changes the state, it will
 //   trigger flipBit() and change the object value that way.
  tmp = value.toInt();
  for(i = 0; tmp; ++i) {
    if(checks.find(i) != checks.end())
      checks[i]->setCheckState(tmp & 1 ? Qt::Checked : Qt::Unchecked);
    tmp >>= 1;
  }
}

void BitString::addCheck(QString text, int bit) {
  if(bit > 31)
    return;

  QCheckBox *check = new QCheckBox(text);
  if(bs & (1 << bit))
    check->setCheckState(Qt::Checked);

  connect(check, SIGNAL(stateChanged(int)), sigmap, SLOT(map()));
  sigmap->setMapping(check, bit);
  checkLayout->addWidget(check, checkCount % height, checkCount / height);
  checks[bit] = check;

  ++checkCount;
}

